Object = require "classic"
require "player"
require "button"
require "wall"

love.window.setTitle("Stomp")

local joysticks = love.joystick.getJoysticks()
Joystick = joysticks[1]
if Joystick then nbbuttons = 4 else nbbuttons = 3 end

death = love.audio.newSource("death.ogg")
menusound = love.audio.newSource("0321.ogg")
background = love.audio.newSource("background.ogg")
death:setLooping(false)
menusound:setLooping(false)
background:setLooping(true)
love.audio.play(background)

crown = love.graphics.newImage("Textures/crown.png")
slowtime = love.graphics.newImage("Textures/slowtime.png")

timealpha = {255, 255, 255, 0}

score1 = 0
score2 = 0
score3 = 0
score4 = 0
highscore = 0
highplayer = 0
menu = true
firstRound = true

windowWidth = love.graphics.getWidth()
windowHeight = love.graphics.getHeight()

font = love.graphics.getFont()

oldGamemode = 1

function love.load()
  if firstRound then gamemode = 0 else gamemode = love.math.random(5) end
  if gamemode == oldGamemode and not firstRound then love.load() end
  oldGamemode = gamemode
  gravity = 1600
  jumpSpeed = -700
  speed = 350
  mult = 1
  if not menu then firstRound = false end
  
  buttons = {}
  players = {}
  alives = {}
  walls = {}
  
  if menu then
    background:setVolume(0.5)
    for i=1,nbbuttons do
      table.insert(buttons, Button())
    end
    
    buttons[1].action = function()
      menu = false
      nbplayers = 1
      love.load()
    end
    buttons[1].color = {255, 0, 0, 255}
    buttons[1].c = buttons[1].color
    buttons[1].text = "1 Player (seriously?)"
    buttons[1].y = 150
    
    buttons[2].action = function()
      menu = false
      nbplayers = 2
      love.load()
    end
    buttons[2].color = {0, 255, 0, 255}
    buttons[2].c = buttons[2].color
    buttons[2].text = "2 Players"
    buttons[2].y = 250
    
    buttons[3].action = function()
      menu = false
      nbplayers = 3
      love.load()
    end
    buttons[3].color = {0, 0, 255, 255}
    buttons[3].c = buttons[3].color
    buttons[3].text = "3 Players"
    buttons[3].y = 350
    
    if #buttons == 4 then
      buttons[4].action = function()
        menu = false
        nbplayers = 4
        love.load()
      end
      buttons[4].color = {255, 255, 0, 255}
      buttons[4].c = buttons[4].color
      buttons[4].text = "4 Players"
      buttons[4].y = 450
    end
  else
    background:setVolume(1)
    for i=1,nbplayers do
      id = i
      table.insert(players, Player())
      table.insert(alives, Player())
      for j,v in ipairs(players) do
        if checkCollision(players[i], v) and i ~= j then 
          table.remove(players, i)
          table.remove(alives, i)
          table.insert(players, Player())
          table.insert(alives, Player())
        end
      end
      if players[i].score > highscore then 
        highplayer = players[i].id 
        highscore = players[i].score
      end
    end
    
    players[1].up = "w"
    players[1].down = "s"
    players[1].left = "a"
    players[1].right = "d"
    if gamemode ~= 3 then players[1].color = {255, 0, 0} end
    
    if nbplayers >= 2 then
      players[2].up = "up"
      players[2].down = "down"
      players[2].left = "left"
      players[2].right = "right"
      if gamemode ~= 3 then players[2].color = {0, 255, 0} end
    end
    
    if nbplayers >= 3 then
      players[3].up = "i"
      players[3].down = "k"
      players[3].left = "j"
      players[3].right = "l"
      if gamemode ~= 3 then players[3].color = {0, 0, 255} end
    end
    
    if nbplayers == 4 then
      players[4].up = "up"
      players[4].down = "down"
      players[4].left = "left"
      players[4].right = "right"
      if gamemode ~= 3 then players[4].color = {255, 255, 0} end
    end
    
    for i=1,4 do
      table.insert(walls, Wall())
      walls[i].id = i
    end
    
    if gamemode == 4 then 
      walls[1].color = {255, 0, 0} 
      walls[1].drawMode = "fill"
    end
    
    walls[2].x = 50
    walls[2].y = 400
    walls[2].width = 200
    walls[2].height = 50
    
    walls[3].x = 550
    walls[3].y = 400
    walls[3].width = 200
    walls[3].height = 50
    
    walls[4].x = 300
    walls[4].y = 250
    walls[4].width = 200
    walls[4].height = 50
  end
end

function love.update(dt)
  if menu then
    for i,v in ipairs(buttons) do
      v:update(dt)
    end
  else
    mindist = 300
    for i,v in ipairs(players) do
      v:update(dt)
      for j,w in ipairs(alives) do
        if v.dead and v.id == w.id then table.remove(alives, j) end
      end
      local stopdetect = false
      for j,w in ipairs(walls) do
        if checkCollision(v, w) then 
          if w.id == 1 and gamemode == 4 then v.dead = true end
          stopdetect = true
          v.y = v.y - v.yspeed * dt * mult
          onx, ony = getSides(v, w)
          if onx then v.x = v.x - v.xspeed * dt * mult end
          v.onGround = true
        else
          if not stopdetect then v.onGround = false end
        end
      end
      for j,w in ipairs(players) do
        if gamemode == 5 and j ~= i and not v.dead and not w.dead then 
          local dist = getDistance(v, w)
          if dist < mindist then mindist = dist end
        end
        if checkCollision(v, w) and not v.dead and not w.dead and v ~= w then
          onx, ony = getSides(v, w)
          if ony == "down" and v.yspeed > 0 and not v.onGround  and gamemode ~= 4 then
            if gamemode == 1 then 
              v.dead = true
              w.yspeed = jumpSpeed / -2
              w.onGround = false
            else
              w.dead = true
              v.yspeed = jumpSpeed / 2
              v.onGround = false
            end
          elseif ony == "up" and v.yspeed < 0 and not v.onGround  and gamemode ~= 4 then
            if gamemode == 1 then
              w.dead = true
              v.yspeed = jumpSpeed / -2
              v.onGround = false
            else
              v.dead = true
              w.yspeed = jumpSpeed / 2
              w.onGround = false
            end
          elseif gamemode ~= 4 then
            v.x = v.x - v.xspeed * dt * mult
            w.x = w.x - w.xspeed * dt * mult
          else
            v.x = v.x + w.xspeed * dt * mult
            w.x = w.x + v.xspeed * dt * mult
          end
        end
      end
    end
    for i,v in ipairs(walls) do
      v:update(dt)
    end
    if #alives == 1 then 
      mult = 0.5 
      background:setPitch(1)
      timealpha[4] = 0
    elseif gamemode == 2 then mult = 2 elseif gamemode == 5 then  
      mult = mindist / 300
      background:setPitch(mindist / 300)
      timealpha[4] = (1 - mindist / 300) * 255
    else 
      mult = 1 
      background:setPitch(1)
    end
    if not love.window.hasFocus() then mult = 0 end
    if #alives == 0 then love.load() end
  end
end

function love.draw()
  if menu then
    for i,v in ipairs(buttons) do
      v:draw()
    end
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.print("Stomp", windowWidth/2 - font:getWidth("Stomp"), 50, 0, 2, 2) 
    love.graphics.print("Music by Eric Matyas, 8-bit Mayhem", windowWidth/2 - font:getWidth("Music by Eric Matyas, 8-bit Mayhem") / 2, 80) 
  else
    for i,v in ipairs(players) do
      v:draw()
    end
    for i,v in ipairs(walls) do
      v:draw()
    end
    if gamemode == 0 then gametext = 'Normal' end
    if gamemode == 1 then gametext = 'Reverse Rules' end
    if gamemode == 2 then gametext = 'Double Speed' end
    if gamemode == 3 then gametext = 'Random Colors' end
    if gamemode == 4 then gametext = "The Floor's Lava" end
    if gamemode == 5 then gametext = "Slow-motion" end
    if #alives > 1 then 
      love.graphics.setColor(255, 255, 255, 255)
      love.graphics.print(gametext, windowWidth/2 - font:getWidth(gametext), 260, 0, 2, 2)
    else
      love.graphics.setColor(players[alives[1].id].color)
      love.graphics.print("Player " .. players[alives[1].id].id .. " won!", windowWidth/2 - font:getWidth("Player " .. players[alives[1].id].id .. " won!"), 260, 0, 2, 2)
    end
    love.graphics.setColor(timealpha)
    love.graphics.draw(slowtime, 0, 0)
  end
  pressed = false
end

function checkCollision(obj, obj2)
  local left = obj.x 
  local up = obj.y
  local right = obj.x + obj.width
  local down = obj.y + obj.height
  
  local left2 = obj2.x 
  local up2 = obj2.y 
  local right2 = obj2.x + obj2.width
  local down2 = obj2.y + obj2.height
  
  return up < down2 and down > up2 and left < right2 and right > left2
end

function love.mousereleased()
  pressed = true
end

function getSides(obj, obj2)
  local ax1 = obj.x 
  local ay1 = obj.y
  
  local bx1 = obj2.x 
  local by1 = obj2.y 
  
  if checkCollision(obj, obj2) then
    local onx = ax1 < bx1 and 'right' or (ax1 > bx1 and 'left' or 'superposed')
    local ony = ay1 < by1 and 'down' or (ay1 > by1 and 'up' or 'superposed')
    return onx, ony
  end
end

function pointInRectangle(pointx, pointy, rectx, recty, rectwidth, rectheight)
    return pointx > rectx and pointy > recty and pointx < rectx + rectwidth and pointy < recty + rectheight
end

function getDistance(obj, obj2)
  local x1 = obj.x
  local y1 = obj.y
  
  local x2 = obj2.x
  local y2 = obj2.y
  
  local dx = x1 - x2
  local dy = y1 - y2
  
  return math.sqrt(dx^2 + dy^2)
end

function math.fact(x)
  local n = 1
  
  for i=1,x do
    n = n * i
  end
  
  return n
end